/***********************************************************************************************************************
 *
 * blueBill Core - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it
 * SCM: https://kenai.com/hg/bluebill~core-src
 *
 **********************************************************************************************************************/
package it.tidalwave.observation;

import javax.annotation.Nonnull;
import java.util.Comparator;
import it.tidalwave.util.As;
import it.tidalwave.util.DefaultFilterSortCriterion;
import it.tidalwave.util.spi.ExtendedFinderSupport;
import it.tidalwave.text.AsDisplayableComparator;

/***********************************************************************************************************************
 *
 * A {@link it.tidalwve.util.Finder Finder} for performing queries on an {@link ObservationSet}. A typical usage is:
 *
 * <pre>
 * ObservationSet observationSet = ...;
 * List&lt;Observable&gt; results = observationSet.find(Observable).
 *                                           from(10).
 *                                           max(20).
 *                                           sort(DISPLAY_NAME, ASCENDING).
 *                                           results();
 * </pre>
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public interface Finder<T> extends ExtendedFinderSupport<T, Finder<T>> 
  {
    public static final SortCriterion BY_DATE = new DefaultFilterSortCriterion<Observation>(
        new Comparator<Observation>()
          {
            public int compare (final @Nonnull Observation o1, final @Nonnull Observation o2)
              {
                return o1.getDate().compareTo(o2.getDate());
              }
          }, "BY_DATE");    
    
    public static final SortCriterion BY_DISPLAY_NAME = new DefaultFilterSortCriterion<As>(AsDisplayableComparator.getInstance(),
      "BY_DISPLAY_NAME");    
    
    public static final SortCriterion BY_OBSERVABLE_DISPLAY_NAME = new DefaultFilterSortCriterion<ObservationItem>(
        new Comparator<ObservationItem>()
          {
            public int compare (final @Nonnull ObservationItem o1, final @Nonnull ObservationItem o2)
              {
                return AsDisplayableComparator.getInstance().compare(o1.getObservable(), o2.getObservable());
              }
          }, "BY_OBSERVABLE_DISPLAY_NAME");  
    
    public static final SortCriterion BY_LOCATION_DISPLAY_NAME = new DefaultFilterSortCriterion<Observation>(
        new Comparator<Observation>()
          {
            public int compare (final @Nonnull Observation o1, final @Nonnull Observation o2)
              {
                return AsDisplayableComparator.getInstance().compare(o1.getLocation(), o2.getLocation());
              }
          }, "BY_LOCATION_DISPLAY_NAME");
  }
